# Django settings for ogra_server project.

import os

# override this ins the stats_settings_local.py
BASE_URL = "/"

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)


# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'Europe/Prague'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
#LANGUAGE_CODE = 'en-us'
LANGUAGE_CODE = 'en'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# supported languages
gettext = lambda s: s
LANGUAGES = (
    ("cs", gettext("Czech")),
    ("en", gettext("English")),           
)

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = False


# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
ABS_CUR_DIR = os.path.abspath(os.path.dirname(__file__))
MEDIA_ROOT = os.path.join(os.path.dirname(ABS_CUR_DIR), "media") + "/"

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = os.path.join(os.path.dirname(ABS_CUR_DIR), "static") + "/"


# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
#    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = '-mnl(-8^mocl4sj*8soth8peya%f1%^x41ge6^j0y&amp;sa@)r+-5'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
#    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.gzip.GZipMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.cache.FetchFromCacheMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'ogra_server.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'ogra_server.wsgi.application'


INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Uncomment the next line to enable the admin:
    'django.contrib.admin',
    'django.contrib.markup',
    # Uncomment the next line to enable admin documentation:
    # 'django.contrib.admindocs',
    'south', # for db schema migrations
    'stats',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.core.context_processors.tz",
    "django.contrib.messages.context_processors.messages",
    "stats.context_processors.piwik"
    )

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'formatters': {
        'classic': {
            'format': '%(levelname)s:%(message)s'
        },
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'console':{
            'level':'DEBUG',
            'class':'logging.StreamHandler',
            'formatter': 'classic',
            #'formatter': 'simple'
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        'stats': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'agora-cache'
    }
}

USE_CACHE_MIDDLEWARE = True # override me in stats_settings_local.py
CACHE_MIDDLEWARE_ALIAS = "default"
CACHE_MIDDLEWARE_SECONDS = 120
CACHE_MIDDLEWARE_KEY_PREFIX = "stats"

# if you use PIWIK, override this in local settings
# URL is *without* the http:// !
PIWIK_URL = ""
PIWIK_ID = None

DEBUG = False
TEMPLATE_DEBUG = DEBUG

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = None

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = None

# directory where to look for arriving updates
INCOMING_DATA_DIR = os.path.join(os.path.dirname(ABS_CUR_DIR), "incoming") + "/"

# Access-Control-Allow-Origin header may be specified here if desired
ACCESS_CONTROL_ALLOW_ORIGIN = None

# import local settings
from stats_settings_local import *

if STATIC_URL is None:
    # it was not set in stats_settings_local
    STATIC_URL = BASE_URL + "static/"
    
if MEDIA_URL is None:
    # it was not set in stats_settings_local
    MEDIA_URL = BASE_URL + "media/"

# process stuff that depends on local settings

MANAGERS = ADMINS

# check if we want to use cache middleware
if not USE_CACHE_MIDDLEWARE:
    new_mws = []
    for mw in MIDDLEWARE_CLASSES:
        if mw not in ('django.middleware.cache.UpdateCacheMiddleware',
                      'django.middleware.cache.FetchFromCacheMiddleware'):
            new_mws.append(mw)
    MIDDLEWARE_CLASSES = tuple(new_mws)

if DEBUG:
    try:
        import devserver
    except ImportError:
        pass
    else:
        INSTALLED_APPS += ('devserver',)
