function ChartPackage(chart_element, error_element, title_element, width, height, language) {
	this.chart_element = chart_element;
	this.error_element = error_element;
	this.title_element = title_element;
	this.width = width ? width : -1;
	this.height = height ? height : -1;
	this.info_element = this.error_element;
	this.chart = null;
	this.language = language;
}

ChartPackage.prototype.set_chart_element = function(el) {
	this.chart_element = el;
};

ChartPackage.prototype.set_error_element = function(el) {
	this.error_element = el;
};

ChartPackage.prototype.set_chart = function(chart_ref, obj) {
	obj.chart = chart_ref;
};

ChartPackage.prototype.show_info = function (text) {
	if (this.info_element != this.chart_element)
		/* do not use it if it is the same as the chart element */
		$(this.info_element).html(text).show(500).delay(5000).hide(500);
}


ChartPackage.prototype.show_error = function (text) {
	$(this.error_element).html(text);
}

ChartPackage.prototype.clean_error = function (text) {
	$(this.error_element).html("");
}

ChartPackage.prototype.show_chart = function (url) {
	$(this.chart_element).empty().addClass("loading");
	jQuery.ajax({
		dataType: "json",
		url: url,
		success: this.handle_chart_reponse.bind(this),
		error: this.handle_chart_response_error.bind(this)  
	});
}	


ChartPackage.prototype.handle_chart_reponse = function(response){
	var el = this.chart_element;
	var data = null;
	eval('data =' + response.data);
	if (response.ok && data != null) {
		this.clean_error();
		/* notes */
		var note_text = "";
		for (var i=0; i < response.notes.length; i++) {
			note_text += "<div>"+response.notes[i]+"</div>";
		}
		if (note_text != "")
			this.show_info(note_text);
		/* the title */
		var title = response.chart_title;
		if (this.title_element != null)
			$(this.title_element).html(title);
		/* the chart */
		var datatable = new google.visualization.DataTable(data, 0.6);
		if (datatable.getNumberOfRows() == 0) {
			/* short-circuit in case no data was found */
			var info_text = response.chart_text;
			if (info_text == null)
				info_text = "No data was received";
			$(el).removeClass("loading");
			$(el).html("<div class='info'><strong>"+info_text+"</strong></div>");
			return
		}
		if (response.value_suffix != null) {
			var formatter = new google.visualization.NumberFormat({suffix: '%'});
			for (var i=1; i<datatable.getNumberOfColumns(); i++)
				formatter.format(datatable, i);
		} else {
			var formatter = new google.visualization.NumberFormat({pattern:'#,###'});
			for (var i=1; i<datatable.getNumberOfColumns(); i++)
				if (datatable.getColumnType(i) == "number")
					formatter.format(datatable, i);
		}
		var vaxis_params = {title: response.dimension_y};
		var haxis_params = {title: response.dimension_x};
		var chart_type = response.chart_type;
		var point_size = null;
		var line_width = null;
		/* sort accorting to column 1 if data is not time dependent */
		if (!response.has_time) {
			datatable.sort([{column: 1, desc: true}]);
		} else {
			if (chart_type == "column_chart" && datatable.getColumnType(0)=='date') {
				/* we need to calculate the x-axis range ourselves */
				var msec_per_day = 24*60*60*1000;
				var min_date = datatable.getValue(0, 0);
				var max_date = datatable.getValue(datatable.getNumberOfRows()-1,
												  0);
				haxis_params.viewWindowMode = 'explicit';
				var diff_days = (max_date.getTime() - min_date.getTime())/msec_per_day;
				var add = diff_days / datatable.getNumberOfRows();
				if (add == 0)
					add = 1;
				var min = new Date();
				var max = new Date();
				min.setTime(min_date.getTime() - add * msec_per_day);
				max.setTime(max_date.getTime() + add * msec_per_day);
				haxis_params.viewWindow = {min: min, max: max};
			}
		}
		/* decide on chart type */
		if (chart_type == "table") {
			//this.chart = null //new google.visualization.Table(el);
		} else if (chart_type == "column_chart") {
			//this.chart = new google.visualization.ColumnChart(el);
		} else if (chart_type == "line_chart") {
			//this.chart = new google.visualization.LineChart(el);
			/* adjust params for discontinuous axis */
			if (datatable.getColumnType(0) != 'date') {
				point_size = 4;
				line_width = 1;
			}
				
		} else if (chart_type == "bar_chart") {
			//this.chart = new google.visualization.BarChart(el);
			/* BarChart is rotated */
			vaxis_params.title = response.dimension_x;
			haxis_params.title = response.dimension_y;
		} else if (chart_type == "pie_chart") {
			//this.chart = new google.visualization.PieChart(el);
		} else {
			//console.log("Error - fallthrough guard hit with value of " + "chart_type = "+chart_type);
		}
		/* adjust bar width */
		var bar_params = {};
		if ((datatable.getNumberOfRows() < 2) &&
			(datatable.getNumberOfColumns() < 3 || response.summable_series))
			bar_params['groupWidth'] = "20%";
		/* client side scaling */
		if (response.client_side_scaling == "log") {
			vaxis_params.logScale = true;
			vaxis_params.format = "#";
		}
		/* decide on width */
		var width = this.width;
		if (width < 0)
			width = this.calculate_available_width();
        
        width = Math.min(1100, width);
        
		var height = this.height;
		if (height < 0)
			height = Math.floor(width / 1.618);

		
		/* show the chart */
		$(el).removeClass("loading");
		if (chart_type == "table") {
			//console.log("Too much data for google table "+datatable.getNumberOfRows());
			this.insert_simple_table(el, datatable);
		} else {
			var params = {
					width: width,
					height: height,
					title: title,
					isStacked: response.summable_series,
					vAxis: vaxis_params,
					hAxis: haxis_params,
					bar: bar_params
				}
			if (line_width != null)
				params.lineWidth = line_width;
			if (point_size != null)
				params.pointSize = point_size;
			if (response.custom_colors != null) {
				params['colors'] = response.custom_colors;

				/*
				var series = [];
				for (var i=0; i<response.custom_colors.length; i++)
					series.push({color: response.custom_colors[i]});
				params.series = series;
				*/
			}
			
			
			// OGRA JS
			// Czech localization
			if (this.language == 'cs') {
				params['lang'] = {
					months: ['leden', 'únor', 'březen', 'duben', 'květen', 'červen', 
						'červenec', 'srpen', 'září', 'říjen', 'listopad', 'prosinec'],
					shortMonths: ['led', 'úno', 'bře', 'dub', 'kvě', 'čer', 
						'čnc', 'srp', 'zář', 'říj', 'lis', 'pro'],
					//numericSymbols: [' tis.', ' mil.', ' mld.'],
					numericSymbols: [' 000', ' 000 000', ' 000 000 000'],
					//decimalPoint: '.',
					thousandsSep: ' ',
				}
			} else {
				params['lang'] = {
					numericSymbols: [' 000', ' 000 000', ' 000 000 000'],
					thousandsSep: ' ',
				}
			}

			// credits
			params['credits_text'] = "CZ.NIC - https://stats.labs.nic.cz/"; 
			
			if (typeof(state_params) == 'undefined') {
				params['credits_href'] = ""; 
			} else {
				var my_url = document.URL.split("?")[0]+"?"+state_params(false);
				params['credits_href'] = my_url; 
			}

			var ogra_chart_type = response.chart_type.substr(0, response.chart_type.length-6);
			
			// correction of line stacking
			if (ogra_chart_type == "line") {
                params.isStacked = false;
            }
            
            var used_library = "high";
            //var used_library = "google";
			
			// getting reference to chart
			if (used_library == "high") {
				params['callback'] = this.set_chart;
				params['callback_args'] = [this];
			}
			
			// creating graph by OGRA
			Ogra.graph(el.id, data, ogra_chart_type, used_library, params);
		}
	} else {
		$(el).removeClass("loading");
		if (response.error != null)
			this.show_error("<b>"+response.error+"</b>");
		else
			this.show_error("<b>No data returned from server.</b>");
	}
}

ChartPackage.prototype.handle_chart_response_error = function(jqXHR, textStatus,
		errorThrown) {
	var el = this.chart_element;
	$(el).removeClass("loading");
	this.show_error("<b>Error occured when fetching data</b>: "+
			errorThrown);
}

ChartPackage.prototype.insert_simple_table = function(el, datatable) {
	/* here we paste the whole text together and the commit
	 * it to the table - it is faster that adding individual elements
	 */
	var table = "<table class='result_table'><thead><tr>";
	for (var j=0; j<datatable.getNumberOfColumns(); j++) {
		table += "<td>"+datatable.getColumnLabel(j)+"</td>";
	}
	table += "</tr></thead><tbody>";
	for (var i=0; i<datatable.getNumberOfRows(); i++) {
		table += "<tr>";
		for (var j=0; j<datatable.getNumberOfColumns(); j++) {
			if (datatable.getColumnType(j) != "number")
				table += "<td class='left'>";
			else
				table += "<td>";
			table += datatable.getFormattedValue(i, j)+"</td>";
		}
		table += "</tr>";
	}
	table += "</tbody></table>";
	$(el).append(table);
}

ChartPackage.prototype.calculate_available_width = function() {
	return $(this.chart_element).innerWidth();
}
