function state_params(exclude_lang) {
	if (exclude_lang === false)
		return $('#chart_params *').not('#lang_dummy_param').serialize();
	else
		return $('#chart_params *').serialize();
}

function show_error(text) {
	chart_package.show_error(text);
}

function clean_error() {
	chart_package.clean_error();
}

function time_processing_changed() {
	var time_processing = $("#time_processing").find(":selected").val();
	if (time_processing.slice(0, 2) != "i-") {
		/* scaling is not compatible with time_processing other than normal */
		$("#scale_select > option[value='normal']").prop('selected', true);
		$("#scale_select").prop('disabled', true);
	} else {
		$("#scale_select").prop('disabled', false);
	}
	update_chart();
}


function setup_series_select() {
	$("#series_select").dropdownchecklist({
		maxDropHeight: 400,
		onComplete: update_chart,
		onItemClick: function(checkbox, selector){
			var enabled = checkbox.prop("checked");
			var current_val = $(checkbox).val();
			if (current_val == "0" || current_val == "-1") {
				/* disable other when all is selected */
				var enable = checkbox.prop("checked") ? 'disabled': '';
				var other_val = current_val == "0" ? "-1" : "0";
				for(var i = 0; i < selector.options.length; i++) {
					if (selector.options[i].value != current_val)
						$(selector.options[i]).prop('disabled', enable);
				}
				$(selector).find("option[value='"+current_val+"']").get(0).
											selected = enabled ? 'selected': '';
				if (enabled) {
					var other = $(selector).find("option[value='"+other_val+"']").get(0);
					if (other != null)
						other.selected = '';
				}
				$(selector).dropdownchecklist("refresh");
			} else if (enabled) {
				/* deselect all if some thing else is selected */
				var desel = $(selector).find("option[value='0']").get(0);
				if (desel != null)
					desel.selected='';
				desel = $(selector).find("option[value='-1']").get(0);
				if (desel != null)
					desel.selected='';
				/* propagate the select */
				$(selector).find("option[value='"+current_val+"']").get(0).selected='selected';
				$(selector).dropdownchecklist("refresh");
			}
		},
		textFormatFunction: function(options) {
			var selector = "#series_select";
			var selected = $(selector).val();
			if (selected == null)
				return "&nbsp;";
			for (var i=0; i<selected.length; i++) {
				if (selected[i] == "0")
					return $(selector).find("option[value='0']").html();
				else if (selected[i] == "-1")
					return $(selector).find("option[value='-1']").html();
			}
			/* normal processing */
			var text = "";
            $(selector).find("option:selected").each(function() {
                if ( text != "" ) { text += ", "; }
                var optCss = $(this).attr('style');
                var tempspan = $('<span/>');
                tempspan.html( $(this).html() );
                if ( optCss == null ) {
                	text += tempspan.html();
                } else {
                	tempspan.attr('style',optCss);
                	text += $("<span/>").append(tempspan).html();
                }
            });
            if (text == "") {
                text = "&nbsp;";
            }
            return text;
		}
	    });
}

function update_series() {
	$("#series_select").prop('disabled', true);
	jQuery.ajax({
		dataType: "json",
		url: "json/series/?"+state_params(),
		success: function(response){
			var data = response.data;
			var series = data.series;
			if (response.ok && data != null) {
				var opts = '';
				for (var i=0; i<series.length; i++) {
					if (series[i][2] == true)
						opts += '<option value="'+series[i][0]+'" selected="selected">'+series[i][1]+'</option>';
					else
						opts += '<option value="'+series[i][0]+'">'+series[i][1]+'</option>';
				}
				$("#series_select").dropdownchecklist("destroy");
				$("#series_select").html(opts);
				$("#series_select").prop('disabled', false);
				setup_series_select();
				/* use the scales from the response */
				var new_scales = "";
				var old_scale = $("#scale_select").val();
				for (i=0; i<data.scales.length; i++) {
					new_scales += "<option value='"+data.scales[i][0]+"'>"+
						data.scales[i][1]+"</option>";
				}
				$("#scale_select").html(new_scales);
				$("#scale_select").val(old_scale);
				update_chart();
			} else {
				show_error("<b>No data returned from server.</b>");
			}
		},
		error:  function(jqXHR, textStatus, errorThrown) {
			show_error("<b>Error occured when fetching data</b>: "+
						errorThrown);
		}});
	
}


function date_range_changed(update) {
	var date_range= $("#date_range").val();
	var current_tp = $("#time_processing").val();
	var diff_ok = true;
	var m_ok = true;
	var y_ok = true;
	if (date_range == "0d") {
		diff_ok = false;
		m_ok = false;
		y_ok = false;
	} else if (date_range.charAt(date_range.length-1) == 'm') {
		y_ok = false;
	}
	// disable incompatible time_processing options
	// diff
	if (!diff_ok)	 
		$("#time_processing > option[value*='diff']").prop('disabled', true);
	else
		$("#time_processing > option[value*='diff']").prop('disabled', false);
	// months
	if (!m_ok)
 		$("#time_processing > option[value='i-1m']").prop('disabled', true);
	else
		$("#time_processing > option[value='i-1m']").prop('disabled', false);
	// years
	if (!y_ok)
 		$("#time_processing > option[value='i-1y']").prop('disabled', true);
	else
		$("#time_processing > option[value='i-1y']").prop('disabled', false);
	// reselect
	if ($("#time_processing > option[value='"+current_tp+"']").prop('disabled'))
		$("#time_processing > option[value='i-1d']").prop('selected', true);
	// update
	if (update !== false) {
		update_chart();
	}
}

function slice_dim_changed() {
	$("#slice_value_select").prop('disabled', true);
	jQuery.ajax({
		dataType: "json",
		url: "json/slices/?"+state_params(),
		success: function(response){
			var data = response.data;
			if (response.ok && data != null) {
				var opts = '';
				for (var i=0; i<data.length; i++) {
					opts += '<option value="'+data[i][0]+'">'+data[i][1]+'</option>';
				}
				$("#slice_value_select").html(opts);
				$("#slice_value_select").prop('disabled', false);
				if (response.slicing_value != null)
					$("#slice_value_select").val(response.slicing_value);
				clean_error();
				update_series();
			} else {
				show_error("<b>No data returned from server.</b>");
			}
		},
		error:  function(jqXHR, textStatus, errorThrown) {
			show_error("<b>Error occured when fetching data</b>: "+
						errorThrown);
		}});
	
}

function ref_date_changed() {
	update_chart();
}

function update_chart() {
	//console.log('updating '+state_params());
	var width = $("#chart_wrapper").innerWidth();
    var height = Math.floor(width / 1.618);
    $("#chart_wrapper").css({"min-height": height});
	update_permalink();
	chart_package.show_chart("json/data/?"+state_params());
}

function download_csv() {
	var url = "csv/?"+state_params();
	window.open(url, '_blank');
}

function update_permalink(event) {
	var href=document.URL.split("?")[0]+"?"+state_params(false);
	$("#chart_permalink").attr({ "href": href });
}
